import threading
import time


class TimeoutThread(threading.Thread):
    """
    class TimeoutThread is used to control the timeout for testcase execution.
    will set executing_test_case's timeout_flag as "True" after timeout.

    @author: Wang Lin
    """

    def __init__(self, executing_test_case):
        threading.Thread.__init__(self)
        self.executing_test_case = executing_test_case

    def run(self):
        # here sleep 0.1 second to make sure the run_testcase_thread.StartTestcaseThread(executing_testcase) executed,
        # because the testcase can set its own timeout.  eg: self.timeout = 100
        time.sleep(0.1)
        # print(self.executing_test_case.timeout)
        time.sleep(self.executing_test_case.timeout)
        self.executing_test_case.timeout_flag = True



"""
update_config.py is used to update the settings/__init__.py from the jenkins job execution.
please copy this module to your project if needed.

the structure of your project should be like this:
your_project_folder
|----  settings/          # settings will be auto-generated by lintest if doesn't exists
|----  common/            # the common libraries
|----  tests/             # all the testcases must be under "tests" package
|----  run.py             # the entry of lintest
|----  update_config.py   # used to update settings/__init__.py for jenkins job execution

e.g:
before update, the content of settings/__init__.py:
----------------------
TESTCASE_TIMEOUT = 120
DEBUG_RUN = True
RERUN_FLAG = False
----------------------

++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
run CMD: "python update_config.py TESTCASE_TIMEOUT=10 debug_run=False rerun_flag=True"
++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

after update, the content of settings/__init__.py:
---------------------
TESTCASE_TIMEOUT = 10
DEBUG_RUN = False
RERUN_FLAG = True
---------------------

Note:
    the argv must be like "key=val key2=val2", each argv split by white-space and the key is case-insensitive

@author: Wang Lin
"""

import sys
import os

os.environ["LIN_TEST_PROJECT_PATH"] = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

from .get_project_info import get_project_info

project_info = get_project_info()

settings_file_path = project_info.project_path + os.sep + "settings" + os.sep + "__init__.py"

argv_list = sys.argv[1:]

if len(argv_list) > 0:
    for argv in argv_list:
        if "=" not in argv:
            raise BaseException("the argv must like: key=val")

    argv_dict = {}
    updated_argv = []

    for argv in argv_list:
        argv_dict[argv.split("=")[0]] = argv.split("=")[1]

    with open(settings_file_path, "r") as settings_file:
        contents = settings_file.readlines()
        i = 0

        while i < len(contents):
            line = contents[i]
            line_remove_whitespace = line.replace(" ", "")

            for argv in argv_dict.keys():
                if line_remove_whitespace.split("=")[0].lower() == argv.lower():
                    contents[i] = "%s = %s \n" % (line_remove_whitespace.split("=")[0], argv_dict[argv])
                    print("UPDATE in settings" + os.sep + "__init__.py: %s " % contents[i])
                    updated_argv.append(argv)
                    break
            i += 1

        with open(settings_file_path, "w") as settings_new_file:
            settings_new_file.writelines(contents)

    if len(updated_argv) < len(argv_list):
        argv_list_not_found = []

        for argv in argv_dict.keys():
            if argv not in updated_argv:
                argv_list_not_found.append(argv)

        print("***************** WARNING: below argv not found in settings/__init__.py *****************")
        print(argv_list_not_found)



"""
run.py is an entry to lintest automation framework.
please copy this module to your project.
the structure of your project should be like this:
your_project_folder
|----  settings/          # settings will be auto-generated by lintest if doesn't exists
|----  common/            # the common libraries
|----  tests/             # all the testcases must be under "tests" package
|----  run.py             # the entry of lintest

@author: Wang Lin
"""

import os
import sys
import traceback


os.environ["LIN_TEST_PROJECT_PATH"] = os.path.dirname(os.path.abspath(__file__))
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

try:
    os.system(
        "pip3 install --upgrade --extra-index-url http://47.117.134.28/simple/ lintest --trusted-host 47.117.134.28")
    from lintest import main
except ImportError:
    traceback.print_exc()

main.run_with(sys.argv)



import traceback

from .base_testcase import BaseTestCase
from .selenium_helper import SeleniumHelper
from selenium.webdriver.remote.webdriver import WebDriver


class UITestCase(BaseTestCase):
    """
    UITestCase is the superclass for all the UI Test cases

    @author: Wang Lin
    """

    def __init__(self):
        super().__init__()
        self.browser = WebDriver

    def create_browser_driver(self):
        self.browser = SeleniumHelper.open_browser(self)

    def close_browser(self):
        browser_list = []

        for key, val in self.__dict__.items():
            if isinstance(val, WebDriver):
                browser_list.append(key)

        for browser in browser_list:
            webdriver = getattr(self, browser)
            try:
                webdriver.delete_all_cookies()
                webdriver.quit()
            except BaseException:
                traceback.print_exc()



# '''
# auto generate a Suite.txt file which according to the Robot Framework Require format.
#
# mainly include three steps
#     step1: collect all the test cases which should be executed
#     step2: fetch the related Library for each test case and generate the Suite.txt file
#     step3: execute command "pybot Suite.txt"
#
# @author: Wang Lin
# '''
# import os
# import time
# import datetime
# from tests import tests_testcases
#
# def generate_suite(testcase_list, suite_name, suit_directory):
#     file_obj = open("%s\%s" % (suit_directory, suite_name), "w")
#     suite_header = '''*** Settings ***
# Test Template'''
#     file_obj.write(suite_header)
#
#     # add all testcase's related Libraries
#     for testcase in testcase_list:
#         testcase_module = testcase.__module__
#         testcase_module_name = testcase_module.split(".")[-1]
#         new_testcase_module = testcase_module.replace(".", "\\")
#         testcase_related_lib = new_testcase_module + ".py"
#         testcase_related_lib = testcase_related_lib.replace("\\", "/")
#         testcase_name = testcase.__name__
#         all_text = '''
# Library           %s
# Library           %s.%s''' % (testcase_related_lib, testcase_module_name, testcase_name)
#         file_obj.write(all_text)
#     file_obj.write("\n\n*** Test Cases ***")
#
#     # add all test cases
#     for testcase in testcase_list:
#         testcase_module = testcase.__module__
#         module_items = testcase_module.split(".")
#         testcase_module_name = module_items[-1]
#         position = len(module_items) - 2
#         feature = module_items[position]
#         testcase_name = testcase.__name__
#         all_text = '''
# %s
#     [Tags]     %s
#     [Setup]    %s.%s.create browser driver
#     %s.%s.runTest
#     [Teardown]    %s.%s.close browser
#     ''' % (
#         testcase_name, feature, testcase_module_name, testcase_name, testcase_module_name, testcase_name, testcase_module_name, testcase_name)
#         file_obj.write(all_text)
#
#     file_obj.close()
#
#     # sleep 2 seconds make sure the Suite.txt have been generated completed
#     time.sleep(2)
#     cmd = r'''pybot '''
#     cmd = cmd + ''' %s\%s''' % ("robot_test_suite", suite_name)
#     print cmd
#
# if __name__ == "__main__":
#     testcase_list = tests_testcases
#     suite_name = "Suite.txt"
#     suite_directory = "..\\robot_test_suite"
#
#     begin_time = datetime.datetime.now()
#     print("execution begin at time: %s" % (begin_time))
#
#     path = os.path.dirname(os.path.abspath(__file__))
#
#     generate_suite(testcase_list, suite_name, suite_directory)



# this module is used to log the memory usage
# @author: Wang Lin

# import time
# import psutil
# import os
# from . import get_project_info
#
# project_info = get_project_info.get_project_info()


def log_memory_usage(status_desc, testcase=None):
    # 暂时不开启
    return

    # memory_usage_log_file_path = project_info.output_folder + os.sep + "memory_usage.log"
    #
    # with open(memory_usage_log_file_path, "a") as memory_usage_log:
    #     current_time = time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime())
    #     memory_usage_log.write("Time: " + current_time + " |  Memory Usage: " +
    #                            str(psutil.virtual_memory()) + " | CPU Usage: " + str(psutil.cpu_percent()) + "%s" % (" | Testcase: " + str(testcase).split(" ")[0] if
    #                                                                   testcase is not None else "") + " | In Status(%s)" % status_desc + os.linesep)


if __name__ == "__main__":
    log_memory_usage()



from xml.etree.ElementTree import iterparse


def parse_xml(filename, path):
    path_parts = path.split('/')
    doc = iterparse(filename, ('start', 'end'))

    # # Skip the root element
    next(doc)

    tag_stack = []

    for event, elem in doc:
        if event == 'start':
            tag_stack.append(elem.tag)
        elif event == 'end':
            if tag_stack == path_parts:
                yield elem

            try:
                tag_stack.pop()
            except IndexError:
                pass


data = parse_xml('xunitresults.xml', 'testcase')

print(data)
for d in data:
    system_out = d.findtext("system-out")
    print("--- system-out:\n")
    print(system_out)
    print("===" * 30)
    failure = d.findtext("failure")
    print("--- failure:\n")
    print(failure)
    print("---" * 30)
    print("\n")

import xml.etree.cElementTree as ET

tree = ET.ElementTree(file="xunitresults.xml")
root = tree.getroot()
# print(root.tag)
print(root.attrib)


#todo: not save the logs into db after each execution done?
# how about: scan the output folder and fetch all the info then genreate the xml file than conver the xml into DB ?
#  benifit: if there are 100 testcases need to be run, but the execution interruped when 90 testcases's execution done,
#  than we can only run the left 10 testcases, and conver the xml into DB.(set the execution_id manually for the second run?)



import logging
import time
import traceback
# import threading

from .logged_requests import LoggedRequests
from pprint import pformat
from pprint import pprint


try:
    import settings
except ImportError:
    traceback.print_exc()

class BaseTestCase(object):
    """
BaseTestCase is the superclass for all the backend(Non UI) and frontend(UI) Testcases and Mobile Testcases
@author: Wang Lin
"""

    # TestEngineCaseInput 即 TestEngineCaseInput from github action（ test-engine目前只支持单线程）
    TestEngineCaseInput = {}

    # GlobalData 是全局数据对象（各个不同的thread发起的不同的testase直接共享该数据）
    GlobalData = {}

    priority = None

    def __init__(self):
        self.pprint = pprint
        self.pformat = pformat
        self.logger = logging
        self.requests = LoggedRequests

        self.testcase_id = None

        self.TestEngineCaseInput = BaseTestCase.TestEngineCaseInput

        # TestEngineTestEngineCaseOutput: case由test-engine触发执行结束后call-back时传入的参数（ test-engine目前只支持单线程）
        self.TestEngineCaseOutput = {}

        self.GlobalData = BaseTestCase.GlobalData

        # # todo: 此处用于 构建多线程时 每个线程独立的 TestEngineCaseInput(线程id作为key)
        # print("===================+++++++++++++==========threading.currentThread().ident")
        # print(threading.currentThread().ident)

        # self.CaseData 用于记录 case执行过程中产生的数据（多数情况用于 某个 testcase chain中, 某个Testcase中有调用了 另外一个Testcase.run_test() 则此case 被定义一个 TestcaseChain）
        # eg:
        # from lintest.api_testcase import APITestCase
        # from tests.api.testcase_demo.testcase1 import Testcase1
        # from tests.api.testcase_demo.testcase2 import Testcase2
        #
        # class Testcase3(APITestCase):
        #     tag = 'case3'
        #
        #     def run_test(self):
        #         # 如下调用方式 Testcase1 & Testcase2 的log 不会 被保存到 Testcase3 的log 中
        #         # Testcase1().run_test()
        #         # Testcase2().run_test()
        #         # print(self.CaseData)
        #
        #         # 如下调用方式 Testcase1 & Testcase2 的log  会  被保存到 Testcase3 的log 中
        #
        #         # 比如： TestAdd1.run_test(self)中会设置 username, self.CaseData['username'] = 'lin'
        #         Testcase1.run_test(self)
        #
        #         # TestAdd2.run_test(self)中会设置 password, self.CaseData['password'] = res['password'], 其中 res可能是某个方法或方法的返回值
        #         Testcase2.run_test(self)
        #
        #         # 此时可以在 TestAdd3 直接通过 self.CaseData 获取 对应的 username & password
        #         self.logger.info(self.CaseData)
        #         self.logger.info(self.CaseData['username'])
        #         self.logger.info(self.CaseData['password'])
        #
        #         # login(self.CaseData['username']), self.CaseData['password']))
        self.CaseData = {}

    # now timeout only support integer seconds, default value is 1200 seconds
    timeout = getattr(settings, "TESTCASE_TIMEOUT", 1200)

    def sleep(self, sleep_seconds):
        time.sleep(sleep_seconds)

    def run_test(self):
        """
        this method must be implement by all backend test cases,
        each test case must encapsulate its business logic into a specific method which called run_test():
        def run_test(self):
            ...
        """
        raise NotImplementedError("---subclass: %s must implement this method: run_test(self)!" % self)

    def teardown(self):
        """
        def teardown(self):
            ...
        """
        pass

    def setup(self):
        """
        def teardown(self):
            ...
        """
        pass

    # def tearDown(self):
    #     """
    #     def teardown(self):
    #         ...
    #     """
    #     pass

    # def get_testcase_id(self):
    #     return self.testcase_id

    # @abc.abstractmethod
    # def set_testcase_id(self):
    #     """
    #     self.testcase_id = "real testcase_id"
    #     """
    #     pass

    # # todo: really need this?
    # def set_testcase_id(self):
    #     set_testcase_id_msg = """
    #     Note: you should add set_testcase_id() in your testcase like below:
    #     def set_testcase_id():
    #         testcase_id = "1"
    #     """
    #     framework_logger.error(set_testcase_id_msg)

    def assert_equals(self, actual_val, expect_val):
        try:
            assert actual_val == expect_val
            self.logger.info("assert actual_val: %s == expect_val: %s  ====== Passed" % (actual_val, expect_val))
        except AssertionError as e:
            e.args += ('expected value is %s' % expect_val, 'actual_val is %s' % actual_val)
            self.logger.info("assert_equals(%s, %s)  ------ Failed" % (actual_val, expect_val))
            raise

    def assert_is_not_none(self, actual_val):
        try:
            assert actual_val is not None
            self.logger.info("assert %s is not None  ====== Passed" % (actual_val))
        except AssertionError as e:
            e.args += ('expected value is not None', 'actual_val is %s' % actual_val)
            self.logger.info("assert_is_not_none(%s)  -- Failed" % (actual_val))
            raise



# """
# This module is used to:
#  create the framework_logger instance
#
# @author: Wang Lin
# """
# # import logging
# # import os
# # import sys
# # from . import get_project_info
# #
# #
# # execution_log_file = get_project_info.get_project_info().project_path + os.sep + "output" + os.sep + get_project_info.ProjectInfo.start_time_for_output + os.sep + "execution.log"
# #
# # # Log file location
# # logfile = execution_log_file
# # # Define the log format
# # log_format = (
# #     '%(asctime)s [%(threadName)-12.12s] %(levelname)s %(filename)s %(lineno)d: %(message)s')
# #
# # # Define basic configuration
# # logging.basicConfig(
# #     # Define logging level
# #     level=logging.DEBUG,
# #     # Declare the object we created to format the log messages
# #     format=log_format,
# #     # Declare handlers
# #     handlers=[
# #         logging.FileHandler(logfile),
# #         # logging.StreamHandler(sys.stdout),
# #     ]
# # )
# #
# # framework_logger = logging
#
#
# import logging
#
#
# # Todo framework_logger 需要重构！ 之前的版本，会把所有的case的log 都写到 execution_log_file 文件中去（随着日志越来越多，文件越来越大，会很慢）
# class TodoLog:
#     @staticmethod
#     def info(obj):
#         pass
#
#     @staticmethod
#     def debug(obj):
#         pass
#
#     @staticmethod
#     def warn(obj):
#         pass
#
#     @staticmethod
#     def error(obj):
#         pass
#
#
# framework_logger = TodoLog



import os
import socket
import datetime
import traceback

try:
    import lintest
except ImportError:
    traceback.print_exc()


def convert_to_seconds(t):
    try:
        s_time = str(t)
        # print("start to covert to seconds:" + s_time)
        if "." in s_time:
            s_time, ms = s_time.split(".")
            ms = "0." + ms
        else:
            ms = 0
        h, m, s = s_time.split(":")
        h, m, s = int(h), int(m), int(s)
        s_time = h * 3600 + m * 60 + s + float(ms)
        s_time = str(s_time)
        return s_time
    except BaseException:
        traceback.print_exc()
        return t


def generate_xunit_result(passed_testcases, failed_testcases, output_folder, begin_time):
    end_time = datetime.datetime.now()
    execution_time = end_time - begin_time

    with open(output_folder + os.sep + "xunitresults.xml", "w", encoding='utf-8') as xml_file:
        pass_count = len(passed_testcases)
        fail_count = len(failed_testcases)

        xml_file.write(
            "<testsuite failures='%s' tests='%s' name='Automation Run' hostname='%s' time='%s' timestamp='%s' type='lintest_%s'>\n" % (
                fail_count, pass_count + fail_count, socket.getfqdn(), execution_time, begin_time,
                lintest.__version__))

        for failed_testcase in failed_testcases:
            xml_file.write("    <testcase classname='%s' name='%s' time='%s' status='fail'>\n" % (
                failed_testcase.__module__, failed_testcase.__class__.__name__,
                failed_testcase.execution_time))

            # the value of xml's attribute must not contains the "<" or ">" character, here replace "<" to "&lt;" and
            # replace ">" to "&gt;" and replace "&" to "&amp;" and replace " to '
            xml_file.write(
                "        <failure message=\"%s\">\n" % failed_testcase.exception_info_for_xml_report.replace('"',
                                                                                                             "'").replace(
                    "<", "&lt;").replace(">", "&gt;").replace("&", "&amp;"))

            xml_file.write(
                failed_testcase.traceback_info.replace('"', "'").replace("<", "&lt;").replace(">", "&gt;").replace("&",
                                                                                                                   "&amp;"))
            xml_file.write("        </failure>\n")

            xml_file.write("        <system-out>\n")

            with open(failed_testcase.logfile_full_name, "r", encoding='utf-8') as f:
                full_log = f.read()
                xml_file.write(
                    full_log.replace('"', "'").replace("<", "&lt;").replace(">", "&gt;").replace("&", "&amp;"))

            xml_file.write("        </system-out>\n")

            xml_file.write("    </testcase>\n\n")

        for passed_testcase in passed_testcases:
            xml_file.write("    <testcase classname='%s' name='%s' time='%s' status='pass'>\n" % (
                passed_testcase.__module__, passed_testcase.__class__.__name__,
                passed_testcase.execution_time))

            xml_file.write("        <system-out>\n")
            with open(passed_testcase.logfile_full_name, "r", encoding='utf-8') as f:
                full_log = f.read()
                xml_file.write(
                    full_log.replace('"', "'").replace("<", "&lt;").replace(">", "&gt;").replace("&", "&amp;"))

            xml_file.write("        </system-out>\n")
            xml_file.write("    </testcase>\n\n")

        xml_file.write("</testsuite>")



import platform
import subprocess


def get_platform_info():
    """
    Returns: the OS name & release version

    @author: Wang Lin
    """
    if platform.system() == "Windows":

        system_info = subprocess.check_output("systeminfo", shell=True, universal_newlines=True)

        for line in system_info.splitlines():
            if "OS Name:" in line:
                return line.split("OS Name:")[1].strip()

    elif platform.system() == "Darwin":
        system_info = subprocess.check_output("sw_vers", shell=True, universal_newlines=True)
        # print(type(system_info))

        for info in system_info.splitlines():
            if "ProductName" in info:
                product_name = info.split(":")[1].strip()

            if "ProductVersion" in info:
                product_version = info.split(":")[1].strip()

        return product_name + " " + product_version

    else:
        return platform.system() + " " + platform.release()

def get_xcode_version():
    if platform.system() == "Darwin":
        xcode_info = subprocess.check_output("xcodebuild -version", shell=True, universal_newlines=True)

        for info in xcode_info.splitlines():
            if "xcode" in info.lower():
                xcode_version = info.split(" ")[1]
                return xcode_version


def get_ios_platform_version():
    if platform.system() == "Darwin":
        ios_platform_version = subprocess.check_output("xcrun simctl list devices", shell=True, universal_newlines=True)

        for info in ios_platform_version.splitlines():
            if "iOS" in info:
                ios_platform_version = info.split("iOS")[1].strip().split('-')[0]
                return ios_platform_version.strip()


if __name__ == "__main__":
    print(get_platform_info())



"""
database_helper.py is used to save the lintest's execution's logs

@author: Wang Lin
"""
import os
import socket
import traceback
import pymysql


def insert_tc_log(project_name, execution_id, tc_id, tc_name, file, testcase_full_path, execution_time, status,
                  error_info="", execution_log="", tags="", package_list="", jenkins_job_name="",
                  rerun_flag=False, error_message="", screenshot_list_for_db="", testcase_start_time=""):
    try:
        conn = pymysql.connect(user='root', password='password', database='testdb')
        cousor = conn.cursor()
        execution_log = execution_log.replace("'", '"') if execution_log is not None else ""
        error_info = error_info.replace("'", '"') if error_info is not None else ""

        insert_tc_sql = """
        insert into execution_logs(project_name, execution_id, tc_id, tc_name, file, testcase_full_path,
                    execution_time, status, error_info, execution_log, tags, package_list, jenkins_job_name, 
                    rerun_flag, error_message, screenshots,testcase_start_time)
        VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', %s, '%s', '%s', '%s')
        """ % (project_name,
               execution_id,
               tc_id,
               tc_name,
               file,
               testcase_full_path,
               execution_time,
               status,
               conn.escape_string(error_info) if error_info else "",
               conn.escape_string(execution_log) if execution_log else "",
               tags,
               package_list,
               jenkins_job_name,
               rerun_flag,
               conn.escape_string(error_message) if error_message else "",
               screenshot_list_for_db,
               testcase_start_time)

        print("-------- executed done insert_tc_sql: %s" % insert_tc_sql)
        cousor.execute(insert_tc_sql)
    except BaseException:
        traceback.print_exc()
        print(traceback.format_exc())
    finally:
        conn.commit()
        cousor.close()
        conn.close()


def insert_execution_summary_log(execution_id,
                                 environment=None,
                                 os=None,
                                 automation_framework_version=None,
                                 total_execution_time=None,
                                 jenkins_job_name=None,
                                 project_name=None,
                                 total_testcases_count=None,
                                 pass_testcases_count=None,
                                 fail_testcases_count=None,
                                 rerun_flag=0,
                                 client=None):
    try:
        conn = pymysql.connect(user='root', password='password', database='testdb')
        cousor = conn.cursor()
        insert_execution_summary_sql = """
        insert into execution_summary(execution_id, environment, os, automation_framework_version, total_execution_time,
                    jenkins_job_name, project_name,total_testcases_count, pass_testcases_count, fail_testcases_count, 
                    rerun_flag, client)
        VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s')
        """ % (execution_id,
               environment,
               os,
               automation_framework_version,
               total_execution_time,
               jenkins_job_name,
               project_name,
               total_testcases_count,
               pass_testcases_count,
               fail_testcases_count,
               rerun_flag,
               socket.getfqdn()
               )

        print("insert_execution_summary_sql: %s" % insert_execution_summary_sql)
        cousor.execute(insert_execution_summary_sql)
    except BaseException:
        traceback.print_exc()
        print(traceback.format_exc())
    finally:
        conn.commit()
        cousor.close()
        conn.close()


def save_execution_log(passed_testcases, failed_testcases, output_folder, project_name, jenkins_job_name="", rerun_flag=False):
    try:
        conn = pymysql.connect(user='root', password='password', database='testdb')
        cursor = conn.cursor()

        for testcase in (passed_testcases + failed_testcases):
            error_info = testcase.traceback_info.replace("'",
                                                         '"') if hasattr(testcase,
                                                                         "traceback_info") else None

            error_message = testcase.error_message_for_db if hasattr(testcase,
                                                                     "error_message_for_db") else ""

            # todo: only for UI testcase need screenshot
            screenshot_list_for_db = ""
            # todo: here should get the configure for save screenshot path.
            for screenshot_path in testcase.screenshot_path_list_for_db:
                screenshot_list_for_db += output_folder.split(os.sep)[-1] + screenshot_path.strip().replace(
                    output_folder, "") + ";"
                print(screenshot_list_for_db)

            insert_tc_sql = """
                    insert into execution_logs(project_name, execution_id, tc_id, tc_name, file, testcase_full_path,
                                execution_time, status, error_info, execution_log, tags, package_list, jenkins_job_name, 
                                rerun_flag, error_message, screenshots,testcase_start_time)
                    VALUES ('%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', %s, '%s', '%s', '%s')
                    """ % (project_name,
                           output_folder.split(os.sep + "output" + os.sep)[1],
                           getattr(testcase, "testcase_id", "None"),
                           testcase.__class__.__name__,
                           testcase.__module__,
                           testcase.__module__ + "." + testcase.__class__.__name__,
                           testcase.execution_time,
                           "pass" if testcase in passed_testcases else "fail",
                           conn.escape_string(error_info) if error_info else "",
                           conn.escape_string(testcase.execution_log) if testcase.execution_log else "",
                           testcase.tag,
                           testcase.packages,
                           jenkins_job_name,
                           testcase.rerun_tag,
                           conn.escape_string(error_message) if error_message else "",
                           screenshot_list_for_db,
                           testcase.testcase_start_time)

            print("-------- executed done insert_tc_sql: %s" % insert_tc_sql)
            cursor.execute(insert_tc_sql)
    except BaseException:
        traceback.print_exc()
        print(traceback.format_exc())
    finally:
        conn.commit()
        cursor.close()
        conn.close()



"""
This module is used to:
  auto generate the testcase list by the .csv & related .py file


如果需要使用 dataSet(目前只支持CSV格式）,请按照如下方式定义：
    1. 定义一个 csv文件，该文件中必须包含id列
    2. 在csv文件的同级目录下定义一个 和csv文件同名的.py文件，py文件中必须定义一个 run_test(self, param...)方法
    参考如下：
       1. testLogin.csv 文件如下：
           id,channel,username,password
            1,web,"admin1","admin1"
            2,web,admin2,"admin2"
            3,web,user1,user2
            4,app,admin1,admin1
            5,app,admin2,admin2

       2. testLogin.py文件内容如下：
            from common.test_login import login # 引入的具体业务逻辑处理方法

            def run_my_test(self, username, password, channel): # 定义接受的参数（此处接收的参数都必须定义在csv中--列名）
                self.logger.info("start run_test()...")
                login(self, username, password, channel)

      -----> 如果正确运行后，框架会根据 testLogin.csv & testLogin.py 自动生成符合框架运行标准的 auto_generated_by_csv.py文件（testLogin_case_list_auto_generated_by_csv.py）
        注意：和__init__.py文件类似，每次执行，都会根据相应的 .csv & .py文件自动生成最新的 auto_generated_by_csv.py文件
      testLogin_case_list_auto_generated_by_csv.py 内容如下：

        from lintest.api_testcase import APITestCase
        from tests.api.csv_dataset import testLoginDataSet


        class testLoginDataSet_1(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin1'
                password = 'admin1'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_2(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin2'
                password = 'admin2'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_3(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'user1'
                password = 'user2'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_4(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin1'
                password = 'admin1'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_5(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin2'
                password = 'admin2'

                testLoginDataSet.run_test(self, username, password)

@author: Wang Lin
"""

import os
import csv
import traceback

from . import get_project_info


PROJECT_INFO = get_project_info.get_project_info()
TESTS_PACKAGE_PATH = PROJECT_INFO.project_path + os.sep + PROJECT_INFO.testcase_package_name


def generate_testcase_list_by_csv(file_name, row, is_ui_testcase, tag_name=''):
    py_file_name = file_name.split(".")[0] + ".py"

    # 先 解析 py 文件,检测 文件中是否包含 def run_test(self, param1, param2 ...) 方法定义， 否则提示用户必须按照这种规范来写 dataSet（csv)的testcase
    with open(py_file_name, "r", encoding='utf-8') as py_file_obj:
        has_run_test_func_flag = False

        for line in py_file_obj.readlines():
            if line.startswith("def ") and line.__contains__("run_test") and line.__contains__("(") and \
                    line.__contains__(")") and line.__contains__(":") and not line.strip().startswith("#") \
                    and line.replace(" ", "").startswith("defrun_test(self,"):
                has_run_test_func_flag = True
                break

        if not has_run_test_func_flag:
            errInfo = """
            %s 文件 不符合如下规范！ 请检查更新后重新运行！
            如果需要使用 dataSet(目前只支持CSV格式）,请按照如下方式定义：
            1. 定义一个 csv文件，该文件中必须包含id列
            2. 在csv文件的同级目录下定义一个 和csv文件同名的.py文件，py文件中必须定义一个 run_test(self, param...)方法
            参考如下：
               1. testLogin.csv 文件如下：
                   id,channel,username,password
                    1,web,"admin1","admin1"
                    2,web,admin2,"admin2"
                    3,web,user1,user2
                    4,app,admin1,admin1
                    5,app,admin2,admin2
                    
               2. testLogin.py文件内容如下：
                    from common.test_login import login # 引入的具体业务逻辑处理方法
        
                    def run_my_test(self, username, password, channel): # 定义接受的参数（此处接收的参数都必须定义在csv中--列名）
                        self.logger.info("start run_test()...") 
                        login(self, username, password, channel)
                        
              -----> 如果正确运行后，框架会根据 testLogin.csv & testLogin.py 自动生成符合框架运行标准的 auto_generated_by_csv.py文件（testLogin_case_list_auto_generated_by_csv.py）
                注意：和__init__.py文件类似，每次执行，都会根据相应的 .csv & .py文件自动生成最新的 auto_generated_by_csv.py文件
              testLogin_case_list_auto_generated_by_csv.py 内容如下：
              
                from lintest.api_testcase import APITestCase
                from tests.api.csv_dataset import testLoginDataSet
                
                
                class testLoginDataSet_1(APITestCase):
                    tag = "testLoginDataSet"
                
                    def run_test(self):
                        username = 'admin1'
                        password = 'admin1'
                
                        testLoginDataSet.run_test(self, username, password)
                
                
                class testLoginDataSet_2(APITestCase):
                    tag = "testLoginDataSet"
                
                    def run_test(self):
                        username = 'admin2'
                        password = 'admin2'
                
                        testLoginDataSet.run_test(self, username, password)
                
                
                class testLoginDataSet_3(APITestCase):
                    tag = "testLoginDataSet"
                
                    def run_test(self):
                        username = 'user1'
                        password = 'user2'
                
                        testLoginDataSet.run_test(self, username, password)
                
                
                class testLoginDataSet_4(APITestCase):
                    tag = "testLoginDataSet"
                
                    def run_test(self):
                        username = 'admin1'
                        password = 'admin1'
                
                        testLoginDataSet.run_test(self, username, password)
                
                
                class testLoginDataSet_5(APITestCase):
                    tag = "testLoginDataSet"
                
                    def run_test(self):
                        username = 'admin2'
                        password = 'admin2'
                
                        testLoginDataSet.run_test(self, username, password)
                              
            """ % py_file_name
            raise NotImplementedError(errInfo)

    with open(py_file_name, "r", encoding='utf-8') as py_file_obj:
        contents = py_file_obj.readlines()
        new_file_lines = []
        i = 0

        while i < len(contents):
            line = contents[i]

            if line.startswith("def ") and line.__contains__("(") and line.__contains__(":") and \
                    line.__contains__("run_test") and not line.strip().startswith("#"):
                class_name = py_file_name.split(".")[0].split(os.sep)[-1]
                # class_name = "".join(class_name[:1].upper() + class_name[1:]) # todo: 讨论是否需要把clasName首字母转成大写？
                if is_ui_testcase:
                    new_file_lines.append("\n\n\nclass %s_%s(UITestCase):\n\n" % (class_name, row["id"]))
                else:
                    new_file_lines.append("\n\n\nclass %s_%s(APITestCase):\n\n" % (class_name, row["id"]))
                new_file_lines.append('    tag = "%s, %s"\n\n' % (class_name, tag_name))
                param_list = line.replace(" ", "").split("self,")[1].split(")")[0].split(",")
                line = "    def run_test(self):"
                new_file_lines.append(line)

                temp_params_str = ""
                for param in param_list:
                    new_file_lines.append("\n        %s = '%s'" % (param, row[param]))
                    temp_params_str += ", " + param

                new_file_lines.append(
                    "\n\n        %s.run_test(self%s)" % (py_file_name.split(".")[0].split(os.sep)[-1], temp_params_str))
                break

            i += 1

    with open(file_name.split(".")[0] + "_case_list_auto_generated_by_csv.py", "a") as f:
        f.writelines(new_file_lines)


def auto_generate_testcase_list_from_csv(call_by_doctor=False):
    print("start to auto_generate_testcase_list_from_csv")

    # fetch all the test cases which in tests package and organize test cases
    for directory_path, directory_names, file_names in os.walk(TESTS_PACKAGE_PATH, topdown=True):
        csv_files = []

        directory_path = directory_path.replace(PROJECT_INFO.project_path + os.sep, "")

        for file_name in file_names:
            file_full_name = os.path.join(directory_path, file_name)
            if file_full_name.startswith(PROJECT_INFO.testcase_package_name + os.sep + "android" + os.sep):
                continue

            if file_full_name.split(".")[-1] == "csv":
                csv_files.append(file_full_name.split(".csv")[0].split(os.sep)[-1])

        # 如果存在csv文件 才有必要继续判断是否有对应的python文件
        if len(csv_files) > 0:
            for file_name in file_names:
                if file_name.split(".")[-1] == "py" and file_name != "__init__.py" and\
                        file_name.split(".")[0] in csv_files:
                    has_run_test_flag = False

                    with open(PROJECT_INFO.project_path + os.sep + directory_path + os.sep + file_name.split(".")[
                        0] + ".py", "r", encoding='utf-8') as py_file_obj:

                        for line in py_file_obj.readlines():
                            if line.startswith("def run_test(self,"):
                                has_run_test_flag = True
                                break

                    if not has_run_test_flag:
                        print("py文件:%s 格式不符合要求! --- 不会自动生成对应的testcase文件" % file_name)
                        print("""
如果需要使用 dataSet(目前只支持CSV格式）,请按照如下方式定义：
    1. 定义一个 csv文件，该文件中必须包含id列
    2. 在csv文件的同级目录下定义一个 和csv文件同名的.py文件，py文件中必须定义一个 run_test(self, param...)方法
    参考如下：
       1. testLogin.csv 文件如下：
           id,channel,username,password
            1,web,"admin1","admin1"
            2,web,admin2,"admin2"
            3,web,user1,user2
            4,app,admin1,admin1
            5,app,admin2,admin2

       2. testLogin.py文件内容如下：
            from common.test_login import login # 引入的具体业务逻辑处理方法

            def run_my_test(self, username, password, channel): # 定义接受的参数（此处接收的参数都必须定义在csv中--列名）
                self.logger.info("start run_test()...")
                login(self, username, password, channel)

      -----> 如果正确运行后，框架会根据 testLogin.csv & testLogin.py 自动生成符合框架运行标准的 auto_generated_by_csv.py文件（testLogin_case_list_auto_generated_by_csv.py）
        注意：和__init__.py文件类似，每次执行，都会根据相应的 .csv & .py文件自动生成最新的 auto_generated_by_csv.py文件
      testLogin_case_list_auto_generated_by_csv.py 内容如下：

        from lintest.api_testcase import APITestCase
        from tests.api.csv_dataset import testLoginDataSet


        class testLoginDataSet_1(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin1'
                password = 'admin1'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_2(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin2'
                password = 'admin2'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_3(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'user1'
                password = 'user2'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_4(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin1'
                password = 'admin1'

                testLoginDataSet.run_test(self, username, password)


        class testLoginDataSet_5(APITestCase):
            tag = "testLoginDataSet"

            def run_test(self):
                username = 'admin2'
                password = 'admin2'

                testLoginDataSet.run_test(self, username, password)                      
                        """)
                        break


                    print(file_name + "存在对应的csv文件")
                    try:
                        os.remove(PROJECT_INFO.project_path + os.sep + directory_path + os.sep + file_name.split(".")[
                            0] + "_case_list_auto_generated_by_csv.py")
                    except BaseException:
                        pass

                    with open(PROJECT_INFO.project_path + os.sep + directory_path + os.sep + file_name.split(".")[
                        0] + "_case_list_auto_generated_by_csv.py", "w") as f:
                        f.write("from %s import %s\n" % (directory_path.replace(os.sep, "."), file_name.split(".")[0]))

                    is_ui_testcase = False
                    # 先 解析 py 文件, 判断文件中是否有 self.browser 有则说明需要导入UITestCase
                    with open(PROJECT_INFO.project_path + os.sep + directory_path + os.sep + file_name.split(".")[
                        0] + ".py", "r", encoding='utf-8') as py_file_obj:
                        for line in py_file_obj.readlines():
                            if line.replace(" ", "").startswith("self.browser."):
                                is_ui_testcase = True
                                # 导入 UTTestCase
                                with open(PROJECT_INFO.project_path + os.sep + directory_path + os.sep +
                                          file_name.split(".")[
                                              0] + "_case_list_auto_generated_by_csv.py", "a") as f:
                                    f.write("from lintest.ui_testcase import UITestCase\n")
                                break

                    if not is_ui_testcase:
                        # 导入 UTTestCase
                        with open(PROJECT_INFO.project_path + os.sep + directory_path + os.sep +
                                  file_name.split(".")[
                                      0] + "_case_list_auto_generated_by_csv.py", "a") as f:
                            f.write("from lintest.api_testcase import APITestCase\n")

                    csv_file_full_path = PROJECT_INFO.project_path + os.sep + directory_path + os.sep + \
                                         file_name.split('.')[0] + ".csv"


                    # 解析 py 文件, 判断文件中是否有 tag = "tagName" 有则说明需要 为自动生成的testcase 同时设置用户自定义的 tagName
                    tag_name = ''
                    with open(PROJECT_INFO.project_path + os.sep + directory_path + os.sep + file_name.split(".")[
                        0] + ".py", "r", encoding='utf-8') as py_file_obj:
                        for line in py_file_obj.readlines():
                            if line.replace(" ", "").startswith("tag="):
                                tag_name = line.replace(" ", "").split("=")[1].replace('"', "").replace("'", "").replace("\n", "")
                                break

                    with open(csv_file_full_path) as f:
                        f_csv = csv.DictReader(f)
                        for row in f_csv:
                            py_file_name = file_name.split(".")[0] + ".py"
                            generate_testcase_list_by_csv(csv_file_full_path, row, is_ui_testcase, tag_name=tag_name)


if __name__ == "__main__":
    auto_generate_testcase_list_from_csv()



import threading


class SetRunFlagThread(threading.Thread):
    """
    class SetRunFlagThread is used to set the executing_test_case's run_flag to "True"
    @author: Wang Lin
    """

    def __init__(self, executing_test_case):
        threading.Thread.__init__(self)
        self.executing_test_case = executing_test_case

    def run(self):
        self.executing_test_case.run_flag = True



from .base_testcase import BaseTestCase


class AndroidTestCase(BaseTestCase):
    """
    AndroidTestCase is the base class for all the Android Test cases

    @author: WangLin
    """

    REAL_DEVICE = "_real_device_"
    VIRTUAL_DEVICE = "_virtual_device_"

    def __init__(self):
        self.username_temp = "USERNAME_TEMP"
        self.password_temp = "PASSWORD_TEMP"

        from appium import webdriver
        self.android_driver = webdriver.Remote



import os
from setuptools import setup

setup(
    name='lintest',
    version='1.6.3',
    author='Wang Lin',
    author_email='think_wl@163.com',
    packages=['lintest'],
    install_requires=[
    	"psutil",
    	"requests",
    	"selenium",
    	"selenium-wire",
    	"setuptools",
    	"urllib3",
    	"PyMySQL",
    	"chromedriver_autoinstaller"
    ],
)

# python setup.py sdist --formats=gztar,zip



import subprocess
import traceback

try:
    import settings
except ImportError:
    traceback.print_exc()


def get_adb_devices():
    """
    Returns: the adb devices list

    @author: Wang Lin
    """

    adb_devices = []
    offline_devices = []
    device_name_list = []

    return []  # 暂时不开启 APPIUM

    system_info = subprocess.check_output("adb devices", shell=True, universal_newlines=True)

    for info in system_info.splitlines():
        if info.endswith("device"):
            adb_devices.append(info.split("	")[0])
        elif info.endswith("offline"):
            offline_devices.append(info)

    for device_name in adb_devices:
        if device_name in settings.DEVICE_ID_NAME_DICT.keys():
            device_name_list.append(settings.DEVICE_ID_NAME_DICT.get(device_name))
        else:
            device_name_list.append(device_name)

    return device_name_list


if __name__ == "__main__":
    device_list = get_adb_devices()
    print(device_list)



"""
This module is used to:
 1. check the necessary settings files for lintest Automation Framework
 2. analysis the testcases then give the execute suggestions

@author: Wang Lin
"""
import os
import sys
import importlib
import traceback
import platform

from .get_ios_devices_list import get_ios_device_list
from .get_project_info import get_project_info


project_info = get_project_info()

# SETTINGS_INIT_STRING is used to define the default settings/__init__.py content. if there are no settings/__init__.py found
# in the project. then framework will auto generate settings/__init__.py
SETTINGS_INIT_STRING = """
# settings/__init__.py is used to configure below items for the lintest Automation Framework
#  1. FRAMEWORK_LOGGER_LEVEL & TESTCASE_LOGGER_LEVEL
#  2. Environment (test environment: UAT, QA, LIVE ...)
#  3. TESTCASE_TIMEOUT (After Testcase_Timeout seconds, if the testcase still running then throw TimeoutException)
#  4. RERUN_FLAG (control rerun failed testcases. Default is False)
#  5. DEBUG_RUN (TESTCASE_TIMEOUT will be worked on this mode)
#  6. QUEUE_SIZE (set queue_size as 8 means that there are at most 8 test cases can be executed concurrently)
#  7. CLOSE_BROWSER (auto close the browser after UI Testcase's execution done)
#  8. implicitly_wait which only used for webdriver(browser)
#  9. Configuration for Mobile Test ( update follow your business)
#  10. APPIUM_SERVER_DEFAULT_IP = "127.0.0.1"
#  11.APPIUM_SERVER_DEFAULT_PORT = 4723
#  12.IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST
#  13.ios_desired_capabilities
#  14.ios_implicitly_wait
#  15.android_apk_path
#  16.customized_android_capabilities
#  17.android_implicitly_wait
#  18.DEVICE_ID_NAME_DICT
#  19.ANDROID_TESTCASE_ACCOUNT
#  20.# SaveScreenshotForPassedTestCaseFlag
#  21. Configuration for Browser option ( update follow your business)
#  22. generate_xunit_result (default value is False)

import logging

FRAMEWORK_LOGGER_LEVEL = logging.INFO
TESTCASE_LOGGER_LEVEL = logging.DEBUG


class Environment(object):
    UAT = "uat"
    QA = "qa"
    LIVE = "www"

ENVIRONMENT = Environment.UAT
# ENVIRONMENT = Environment.QA
# ENVIRONMENT = Environment.LIVE

# set Test Case Timeout as 1200 seconds
TESTCASE_TIMEOUT = 1200

# control rerun function, default is True
RERUN_FLAG = True

# if debug test cases locally then keep the browser active after execution done. Default value is False
DEBUG_RUN = False

# set queue_size as 8 means that there are at most 8 test cases can be executed concurrently
QUEUE_SIZE = 8

# auto close browser after the testcase's execution done, Default value is True
CLOSE_BROWSER = True

# SaveScreenshotForPassedTestCaseFlag default value if False
SaveScreenshotForPassedTestCaseFlag = False

# set the webdriver's implicitly_wait default value: 60
implicitly_wait = 60

# configure for Database
if ENVIRONMENT == Environment.UAT:
    DATABASE = {
        "Server": "uat-server",
        "User": "uat-user",
        "Password": "uat-password"
    }

if ENVIRONMENT == Environment.QA:
    DATABASE = {
        "Server": "qa-server",
        "User": "qa-user",
        "Password": "qa-password"
    }

if ENVIRONMENT == Environment.LIVE:
    DATABASE = {
        "Server": "live-server",
        "User": "live-user",
        "Password": "live-password"
    }


# ---------------------- config for mobile test --------------------------
APPIUM_SERVER_DEFAULT_IP = "127.0.0.1"
APPIUM_SERVER_DEFAULT_PORT = 4723

# -------- set IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST --------
# IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST is used to control the priority of ios_desired_capabilities which
# maybe provided in settings and testcase, default value is False.
# 1. if IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST = True
#    then the ios_desired_capabilities in settings will override in testcase.
# 2. if IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST = False
#     or no IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST found in settings
#   then will use the ios_desired_capabilities which provided in testcase.
# 3. if there are only one ios_desired_capabilities provided by settings or testcase,
#    then just use it whatever IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST is True or False.
# 4. if no ios_desired_capabilities provided by both settings and testcase then raise exception
IOS_DESIRED_CAPABILITIES_IN_SETTINGS_FIRST = False

# if there are no ios_desired_capabilities found from testcase, then use below as default
from lintest.ios_testcase import IOSTestCase

# -------- set ios_desired_capabilities --------
ios_desired_capabilities = {
    'app': 'app/lintest.app',  # required, "app" directory must under your project, or provide the absolute path
    # 'platformName': 'iOS', # if not provide, will be auto set by lintest framework
    # 'platformVersion': '10.2', # if not provide, will be auto set by lintest framework
    'deviceName': IOSTestCase.DeviceNameList.iPhone6,  # or 'deviceName': 'iPhone 6'
    # 'udid': 'xxx', # for real device
}

# set the implicitly wait time for ios driver, default value is 60 (in seconds)
ios_implicitly_wait = 60

# -------- set android_apk_path --------
android_apk_path = "here should be the real app path.apk"

# set the implicitly wait time for android driver, default value is 60 (in seconds)
android_implicitly_wait = 60

# here can add the customized android capabilities, eg: 'newCommandTimeout: 300'
customized_android_capabilities = "'newCommandTimeout': 300"

# map the device_name with device_id
DEVICE_ID_NAME_DICT = {
    "device_id1": "device_name1",
    "device_id2": "device_name2",
}

# set the android test account, map the user_name & password
ANDROID_TESTCASE_ACCOUNT = {
    "username1": "password1",
    "username2": "password2",
}

# ---------------------- config for browser option --------------------------
# browser option must be a string list, e.g: BROWSER_OPTION = ["--headless", "--window-size=1280x1024"]
BROWSER_OPTION = []


# ---------------------- config for android option --------------------------
# android option must be a string list, e.g: ANDORID_OPTION = ["--chromedriver-path", "the real chromedriver path"]
ANDORID_OPTION = []

"""

# TESTCASE_TAGS_STRING is used to define the class Tags in testcase_tags.py as a reference.
TESTCASE_TAGS_STRING = """# encoding: utf-8
# define the Tags Class for testcases

class Tags(object):
    '''
    class Tags is used to define all the testcase's tags.
    set the tag for each testcase (tag can be any non-blank string, you can identify it follow your business.
        eg."smoke", "nightly", "ignore", also can be the author "Lin")
        Note: # if one testcase's tag contains "ignore", then it will only be added into ignore_testcase_list, even its
            tag attribute contains other tag name

    there are two ways to set tag for testcase
        1. set tag attribute as a string (each tag name split by "," )
        2. recommend: set tag attribute as a list(each tag name define in this Tags class.)

    class FrontendTestcaseDemo(UITestCase):
        # set tag by way 1, this testcase will be auto added into smoke_testcase_list and nightly_testcase_list
        tag = "smoke, nightly"

        # set tag by way 2(recommend this way), it's easy to know all the tag names in your project and will not type
        # an error tag name, eg. type "nighty" as "nightly")
        # this testcase will be auto added into lin_testcase_list & smoke_testcase_list & todo_testcase_lit
        tag = [Tags.Smoke, Tags.Todo]

        def run_test(self):
            self.logger.debug("load google")
            self.browser.get("https://www.google.com")

    Note: the tag name is Not Case Sensitive, that means tag = "smoke" same with tag = "SMOKE"

    @author: Lin Wang
    '''
    Nightly = "nightly"
    Smoke = "smoke"
    Lin = "lin"
    Ignore = "ignore"
    Todo = "todo"
"""


# auto generate settings package, write SETTINGS_INIT_STRING into settings/__init__.py
def generate_settings():
    os.mkdir(project_info.project_path + os.sep + "settings")
    with open(project_info.project_path + os.sep + "settings" + os.sep + "__init__.py", "w") as settings_init_file:
        settings_init_file.write(SETTINGS_INIT_STRING)


# auto generate class Tags
def generate_testcase_tags():
    with open(project_info.project_path + os.sep + "settings" + os.sep + "testcase_tags.py", "w") as testcase_tag_file:
        testcase_tag_file.write(TESTCASE_TAGS_STRING)


def generate_ios_device_list():
    ios_device_list = get_ios_device_list()
    ios_device_str_list = []
    for device_name in ios_device_list:
        ios_device_str_list.append(device_name.replace(" ", "").replace("(", "").replace(")", "").replace(".", "_").replace("-", "_") + ' = ' + '"' + device_name + '"')

    str = ""
    for device_name_str in ios_device_str_list:
        str  += "    " + device_name_str + "\n"

    IOS_DEVICE_LIST_STRING = """
class IOSDeviceNameList:
%s""" % str

    with open(project_info.project_path + os.sep + "settings" + os.sep + "ios_device_list.py", "w") as ios_device_list:
        ios_device_list.write(IOS_DEVICE_LIST_STRING)


def execute_suggestion(testcase_dict=None):
    """
    after doctor check, Doctor will give the execute suggestions.

    """
    print("""-------- there are 5 ways to execute testcase:
1. run CMD: "python run.py tag_name" (here provide the tag_name, then will execute all the testcase which
           has been set tag as tag_name)
 or
2. run CMD: "python run.py TestCaseClassName" (here provide the testcase's class_name: TestCaseClassName(case-insensitive),
           then will execute testcase TestCaseClassName)
 or
3. run CMD: "python run.py testcase_list_name" (provide the testcase_list_name which is defined in each package's
                                    __init__.py file)
 or
4. run CMD: "python run.py package_name" (provide the package_name such as: tests.backend  or tests or
                                    tests.backend.b1)
 or
5. run CMD: "python run.py tag1 tag2 testcase_list_1 testcase_list_2 TestcaseClassName1 package_name_1" (the parameters
                for run.py can be any combination of the following 4 kinds which separated by a space:
                    1. tag_name  (e.g: "smoke" or "regression"
                    2. testcase's class_name (e.g: "TestBackendB1_1")
                    3. testcase_list_name  (e.g: "tests_backend_testcases")
                    4. package_name (e.g: "tests" or "tests.backend" or "tests.frontend")
                all the testcases which fetched by the given parameters will be added into testcase_list, and will do
                data deduplication for testcase_list.
                for each parameters:
                    first try to find it as a tag name, if no tag name found then try to find it as a TestcaseClassName,
                    if no TestcaseClassName found, then try to find it as a TestcaseList, if no TestcaseList found, then
                    try to find it as a package_name
""")

    if testcase_dict is None:
        from .auto_generate_testcase_list import auto_generate_testcase_list
        testcase_dict = auto_generate_testcase_list(call_by_doctor=True)

    if len(testcase_dict.keys()) == 0:
        print("------ there are no tag name found from all the testcases!\n")
    else:
        print("------ found below tag name from your project:")
        print(testcase_dict.keys())
        print(
            "\n------ the testcase list which group by tag name are under: " + project_info.project_path + os.sep + "tests" + os.sep + "tag_testcases")
        print(
            "\n------ run CMD: 'python main.py --help' to see the Usage() for lintest Automation Test Framework ------\n")


def doctor_check():
    """
        doctor_check is used to check the necessary settings files for lintest Automation Framework.
        will auto generate them if not found.
        1. check settings/__init__.py exists
        2. check settings/testcase_tags.py exists
        3. check class Tags exists in settings/testcase_tags.py
    """

    # 1. check whether your_project/settings/__init__.py exists, auto generate it if not
    if not os.path.exists(project_info.project_path + os.sep + "settings" + os.sep + "__init__.py"):
        print("there are no settings package found in your project")
        generate_settings()
        print(
            "auto generate settings/__init__.py done, you should update related configurations by your business logic")
    print("1. check settings/__init__.py exists done")

    # 2. check whether module your_project/settings/testcase_tags.py exists, auto generate it if not exists
    if not os.path.exists(project_info.project_path + os.sep + "settings" + os.sep + "testcase_tags.py"):
        generate_testcase_tags()
    print("2. check settings/testcase_tags.py exists done")

    # 3. check whether class Tags exists in your_project/settings/testcase_tags.py
    try:
        # if there are no testcase_tags found from project, doctor will auto generate it.
        from settings.testcase_tags import Tags
    except ImportError:
        traceback.print_exc()
        print(
            "Warning: there are no Tags found from your_project" + os.sep + "settings" + os.sep + "testcase_tags.py")
        generate_testcase_tags()
        # print type(sys.modules['settings.testcase_tags'])  # output: <type 'module'>

        # module settings.testcase_tags has been added into sys.modules(sys.modules act as the modules cache) when
        # execute "from settings.testcase_tags import Tags", it means that at the begin even there are no Tags defined
        # in settings.testcase_tags.py but the module settings.testcase_tags still be cached in sys.modules. so here
        # must reload() the module "settings.testcase_tags"
        importlib.reload(sys.modules['settings.testcase_tags'])  # reload() require a module as argument

        # module settings.testcase_tags has been added into sys.modules(sys.modules act as the modules cache) when
        # execute "from settings.testcase_tags import Tags", it means that at the begin even there are no Tags defined
        # in settings.testcase_tags.py but the module settings.testcase_tags still be cached in sys.modules.
        # so here execute exec "from settings.testcase_tags import Tags" still use the cached module(still no Tags
        # defined in module settings.testcase_tags without reload it)
        # exec "from settings.testcase_tags import Tags" # this line not work, must call reload()

        print("Auto generate settings" + os.sep + "testcase_tags.py  done!")
        print(
            "Warning: the file testcase_tags which was auto generated just for reference, you should update the tag" \
            " name follow your business logic!\n")
    print("3. check class Tags exists in settings/testcase_tags.py\n")


def doctor():
    """
    doctor() is used to check the necessary configuration files and auto generate them if not exists.
    will give the execution suggestion after check().
    """
    doctor_check()

    from .auto_generate_testcase_list_from_csv import auto_generate_testcase_list_from_csv
    auto_generate_testcase_list_from_csv()

    from .auto_generate_testcase_list import auto_generate_testcase_list

    testcase_dict_for_tags, testcase_dict_for_classname, testcase_dict_for_package = auto_generate_testcase_list(call_by_doctor=True)

    execute_suggestion(testcase_dict_for_tags)

    # auto generate ios_device_name_list.py if on Mac OS
    if platform.system() == "Darwin":
        pass
        # generate_ios_device_list() # 暂时注释掉，目前先不开启 appium

    # try:
    #     # below codes should be executed after check the configuration files done. if below codes executed at the begin
    #     # of doctor(), will throw exception when there are no Tags found from settings/testcase_tags.py(because
    #     # some testcases maybe reference the the tag name which defined in class Tags)
    #     from .auto_generate_testcase_list import auto_generate_testcase_list
    #
    #     testcase_dict_for_tags, testcase_dict_for_classname, testcase_dict_for_package = auto_generate_testcase_list()
    #
    #     execute_suggestion(testcase_dict_for_tags)
    # except BaseException:
    #     traceback.print_exc()


if __name__ == "__main__":
    doctor()



from .base_testcase import BaseTestCase


class IOSTestCase(BaseTestCase):
    """
    IOSTestCase is the base class for all the IOS Test cases

    @author: Lin.Wang
    """

    def __init__(self):
        from appium import webdriver
        self.ios_driver = webdriver.Remote



import platform
import traceback

import settings
import subprocess
import paramiko
from scp import SCPClient


def scp_report_to_specified_path(source_report_path):
    """
    @author: Wang Lin
    """
    ssh_client = paramiko.SSHClient()
    ssh_client.set_missing_host_key_policy(paramiko.AutoAddPolicy)
    ssh_client.connect(settings.SPECIFIED_REPORT_HOST_IP, settings.SPECIFIED_REPORT_HOST_PORT,
                       settings.SPECIFIED_REPORT_HOST_USERNAME, settings.SPECIFIED_REPORT_HOST_PASSWORD)
    scpclient = SCPClient(ssh_client.get_transport(), socket_timeout=15.0)

    try:
        scpclient.put(source_report_path, settings.SPECIFIED_REPORT_PATH, recursive=True, )
    except FileNotFoundError as e:
        traceback.print_exc()
        print(e)
        print("系统找不到指定文件" + source_report_path)
    else:
        print(
            "文件上传成功 from: " + source_report_path + " ----- to: " + settings.SPECIFIED_REPORT_HOST_PORT + settings.SPECIFIED_REPORT_PATH)
    finally:
        ssh_client.close()



import threading
import traceback

from .base_testcase import BaseTestCase


class StartTestcaseThread(threading.Thread):
    """
    StartTestcaseThread is used to call the testcase's runTest()/run_test() method,
    and will set the testcases's done_flag as "True" after execution done without any exception.

    @author: Wang Lin
    """

    def __init__(self, executing_testcase):
        threading.Thread.__init__(self)
        self.executing_testcase = executing_testcase

    def run(self):
        # 先执行 setup()， 再执行 run_test()

        try:
            # 执行 setup() 之前 先 log TestEngineCaseInput(如果是 test-engine 发起的)
            if BaseTestCase.TestEngineCaseInput:
                self.executing_testcase.logger.info("------ TestEngineCaseInput:")
                self.executing_testcase.logger.info(self.executing_testcase.TestEngineCaseInput)

            if hasattr(self.executing_testcase, "setup"):
                self.executing_testcase.logger.info("------ execute setup() Start")
                self.executing_testcase.setup()
                self.executing_testcase.logger.info("------ execute setup() End")
        except BaseException:
            traceback.print_exc()
            pass

        try:
            if hasattr(self.executing_testcase, "runTest"):
                print("the method runTest() is deprecated, please re-name it as run_test()")
                self.executing_testcase.logger.info("================== runTest() start:")
                self.executing_testcase.runTest()
            else:
                self.executing_testcase.logger.info("================== run_test() start:")
                self.executing_testcase.run_test()
        except BaseException:
            traceback.print_exc()
            self.executing_testcase.exception_info = traceback.format_exc()
        else:
            self.executing_testcase.done_flag = True



"""
module clean_data.py is used to:
1. clear all the __init__.py file which under tests package
2. remove all the .pyc files which under tests package

@author: Wang Lin
"""
import os
import time
import shutil
from .get_project_info import get_project_info


project_info = get_project_info()


# remove tests/android_temp if exists
def remove_android_temp(android_testcase_temp_package_name):
    tests_package_path = project_info.project_path + os.sep + project_info.testcase_package_name
    android_temp_package_path = tests_package_path + os.sep + android_testcase_temp_package_name

    while os.path.exists(android_temp_package_path):
        shutil.rmtree(android_temp_package_path)
        time.sleep(2)


# clear all the __init__.py file which under tests package
def clear_init_file():
    for directory_path, directory_names, file_names in os.walk(
                            project_info.project_path + os.sep + project_info.testcase_package_name, topdown=True):
        for file_name in file_names:
            if "__pycache__" in directory_path:
                continue

            if file_name == "__init__.py":
                with open(directory_path + os.sep + file_name, "w"):
                    print("clear file: %s" % (directory_path + os.sep + file_name))


def remove_pyc():
    # clear all the pyc file which under tests package
    for directory_path, directory_names, file_names in os.walk(
                            project_info.project_path + os.sep + project_info.testcase_package_name,
            topdown=True):
        for file_name in file_names:
            if file_name.endswith(".pyc"):
                file_path = directory_path + os.sep + file_name
                print("remove file: %s" % file_path)
                os.remove(file_path)
